from typing import Optional

from cpl_core.database.context import DatabaseContextABC

from bot_core.configuration.bot_settings import BotSettings
from bot_core.logging.database_logger import DatabaseLogger


class DBConfigService:
    def __init__(self, logger: DatabaseLogger, db_context: DatabaseContextABC):
        self._logger = logger
        self._context = db_context

    def get_st_bot(self) -> Optional[dict]:
        result = self._context.select(
            f"""
            SELECT `Id`, `WaitForRestart`, `WaitForShutdown`
            FROM ST_Bot
        """
        )

        if len(result) == 0:
            return None

        # get first element
        result = result[0]

        result_as_dict = {
            "WaitForRestart": result[1],
            "WaitForShutdown": result[2],
        }

        servers = self._context.select(
            f"""
                SELECT 
                    Servers.`ServerId`,
                    `DiscordServerId`,
                    `MessageDeleteTimer`,
                    `MaxVoiceStateHours`,
                    `XpPerMessage`,
                    `XpPerReaction`,
                    `MaxMessageXpPerHour`,
                    `XpPerOntimeHour`,
                    `XpPerEventParticipation`,
                    `AFKCommandChannelId`,
                    `HelpCommandReferenceUrl`,
                    `HelpVoiceChannelId`,
                    `LoginMessageChannelId`,
                    `ChangedLevelNotificationChannelId`
                FROM ST_Servers
                INNER JOIN Servers ON
                ST_Servers.ServerId = Servers.ServerId;
            """
        )

        for server in servers:
            afk_channels = self._context.select(
                f"""
                SELECT `ChannelId`
                FROM ST_Servers_AFKChannelIds
                WHERE ST_Server = {server[0]};
                """
            )
            afk_channel_ids = []
            for afk_channel in afk_channels:
                afk_channel_ids.append(afk_channel[0])

            ping_urls_result = self._context.select(
                f"""
                SELECT `URL`
                FROM ST_Servers_PingURLs
                WHERE ST_Server = {server[0]};
                """
            )
            ping_urls = []
            for url in ping_urls_result:
                ping_urls.append(url[0])

            admin_roles = self._context.select(
                f"""
                SELECT `RoleId`
                FROM ST_Servers_Permissions
                WHERE ST_Server = {server[0]}
                AND RoleType = 2;
                """
            )
            admin_role_ids = []
            for role in admin_roles:
                admin_role_ids.append(role[0])

            moderator_roles = self._context.select(
                f"""
                SELECT `RoleId`
                FROM ST_Servers_Permissions
                WHERE ST_Server = {server[0]}
                AND RoleType = 1;
                """
            )
            moderator_role_ids = []
            for role in moderator_roles:
                moderator_role_ids.append(role[0])

            result_as_dict[server[1]] = {
                "MessageDeleteTimer": server[2],
                "MaxVoiceStateHours": server[3],
                "XpPerMessage": server[4],
                "XpPerReaction": server[5],
                "MaxMessageXpPerHour": server[6],
                "XpPerOntimeHour": server[7],
                "XpPerEventParticipation": server[8],
                "AFKCommandChannelId": server[9],
                "HelpCommandReferenceUrl": server[10],
                "HelpVoiceChannelId": server[11],
                "LoginMessageChannelId": server[12],
                "ChangedLevelNotificationChannelId": server[13],
                "AFKChannelIds": afk_channel_ids,
                "PingURLs": ping_urls,
                "AdminRoleIds": admin_role_ids,
                "ModeratorRoleIds": moderator_role_ids,
            }

        return result_as_dict

    def get_bot_settings(self) -> BotSettings:
        settings = BotSettings()
        settings.from_dict({})
        return settings

    def create_config_for_guild(self, guild_id: int):
        server = self._context.select(
            f"""
            SELECT `ServerId`
            FROM Servers
            WHERE `DiscordServerId` = {guild_id};
        """
        )[0]

        servers = self._context.select(
            f"""
            SELECT *
            FROM ST_Servers
            WHERE `ServerId` = {server[0]}
        """
        )

        if len(servers) > 0:
            self._logger.fatal(__name__, f"Config for guild {guild_id} already exists")
            return

        self._context.cursor.execute(
            f"""
            INSERT INTO `ST_Servers` (`ServerId`) 
            VALUES
            ({server[0]});
        """
        )

        st_server = self._context.select(
            f"""
            SELECT `Id`
            FROM ST_Servers
            WHERE `ServerId` = {server[0]};
        """
        )[0]
